@@ -1,14 +1,12 @@
 use std::task::{Context, Poll};
-use std::{
-    cell::RefCell, convert::TryFrom, fmt, future::Future, marker, pin::Pin, rc::Rc,
-    time::Duration,
-};
+use std::{cell::RefCell, convert::TryFrom, fmt, future::Future, marker, pin::Pin, rc::Rc};
 
 use ntex::codec::{AsyncRead, AsyncWrite};
 use ntex::framed::WriteTask;
 use ntex::service::{IntoServiceFactory, Service, ServiceFactory};
+use ntex::time::{Millis, Seconds, Sleep};
 use ntex::util::timeout::{Timeout, TimeoutError};
-use ntex::{rt::time::Sleep, util::Either};
+use ntex::util::Either;
 
 use crate::error::{MqttError, ProtocolError};
 use crate::io::{DispatchItem, Dispatcher, State, Timer};
@@ -31,8 +29,8 @@ pub struct MqttServer<Io, St, C: ServiceFactory, Cn: ServiceFactory, P: ServiceF
     max_size: u32,
     max_receive: u16,
     max_qos: Option<QoS>,
-    handshake_timeout: u16,
-    disconnect_timeout: u16,
+    handshake_timeout: Seconds,
+    disconnect_timeout: Seconds,
     max_topic_alias: u16,
     pub(super) pool: Rc<MqttSinkPool>,
     _t: marker::PhantomData<(Io, St)>,
@@ -64,8 +62,8 @@ where
             max_size: 0,
             max_receive: 15,
             max_qos: None,
-            handshake_timeout: 0,
-            disconnect_timeout: 3000,
+            handshake_timeout: Seconds::ZERO,
+            disconnect_timeout: Seconds(3),
             max_topic_alias: 32,
             pool: Rc::new(MqttSinkPool::default()),
             _t: marker::PhantomData,
@@ -87,16 +85,16 @@ where
         > + 'static,
     P: ServiceFactory<Config = Session<St>, Request = Publish, Response = PublishAck> + 'static,
 {
-    /// Set handshake timeout in millis.
+    /// Set handshake timeout.
     ///
     /// Handshake includes `connect` packet and response `connect-ack`.
     /// By default handshake timeuot is disabled.
-    pub fn handshake_timeout(mut self, timeout: u16) -> Self {
+    pub fn handshake_timeout(mut self, timeout: Seconds) -> Self {
         self.handshake_timeout = timeout;
         self
     }
 
-    /// Set server connection disconnect timeout in milliseconds.
+    /// Set server connection disconnect timeout.
     ///
     /// Defines a timeout for disconnect connection. If a disconnect procedure does not complete
     /// within this time, the connection get dropped.
@@ -104,7 +102,7 @@ where
     /// To disable timeout set value to 0.
     ///
     /// By default disconnect timeout is set to 3 seconds.
-    pub fn disconnect_timeout(mut self, val: u16) -> Self {
+    pub fn disconnect_timeout(mut self, val: Seconds) -> Self {
         self.disconnect_timeout = val;
         self
     }
@@ -247,7 +245,7 @@ where
         self,
     ) -> impl ServiceFactory<
         Config = (),
-        Request = (Io, State, Option<Pin<Box<Sleep>>>),
+        Request = (Io, State, Option<Sleep>),
         Response = (),
         Error = MqttError<C::Error>,
         InitError = C::InitError,
@@ -286,7 +284,7 @@ where
         InitError = C::InitError,
     >
     where
-        F: Fn(&mqtt::Connect) -> R + 'static,
+        F: Fn(&Handshake<Io>) -> R + 'static,
         R: Future<Output = Result<bool, C::Error>> + 'static,
     {
         let publish = self.srv_publish.map_init_err(|e| MqttError::Service(e.into()));
@@ -304,7 +302,7 @@ where
             max_topic_alias: self.max_topic_alias,
             max_qos: self.max_qos,
             disconnect_timeout: self.disconnect_timeout,
-            time: Timer::with(Duration::from_secs(1)),
+            time: Timer::new(Millis::ONE_SEC),
             _t: marker::PhantomData,
         }
     }
@@ -316,12 +314,12 @@ fn handshake_service_factory<Io, St, C>(
     max_receive: u16,
     max_topic_alias: u16,
     max_qos: Option<QoS>,
-    handshake_timeout: u16,
+    handshake_timeout: Seconds,
     pool: Rc<MqttSinkPool>,
 ) -> impl ServiceFactory<
     Config = (),
     Request = Io,
-    Response = (Io, State, Rc<MqttShared>, Session<St>, u16),
+    Response = (Io, State, Rc<MqttShared>, Session<St>, Seconds),
     Error = MqttError<C::Error>,
 >
 where
@@ -329,9 +327,9 @@ where
     C: ServiceFactory<Config = (), Request = Handshake<Io>, Response = HandshakeAck<Io, St>>,
     C::Error: fmt::Debug,
 {
-    ntex::apply(
-        Timeout::new(Duration::from_millis(handshake_timeout as u64)),
-        ntex::fn_factory(move || {
+    ntex::service::apply(
+        Timeout::new(Millis::from(handshake_timeout)),
+        ntex::service::fn_factory(move || {
             let pool = pool.clone();
 
             let fut = factory.new_service(());
@@ -339,18 +337,21 @@ where
                 let service = fut.await?;
                 let pool = pool.clone();
                 let service = Rc::new(service.map_err(MqttError::Service));
-                Ok::<_, C::InitError>(ntex::apply_fn(service, move |io: Io, service| {
-                    handshake(
-                        io,
-                        None,
-                        service.clone(),
-                        max_size,
-                        max_receive,
-                        max_topic_alias,
-                        max_qos,
-                        pool.clone(),
-                    )
-                }))
+                Ok::<_, C::InitError>(ntex::service::apply_fn(
+                    service,
+                    move |io: Io, service| {
+                        handshake(
+                            io,
+                            None,
+                            service.clone(),
+                            max_size,
+                            max_receive,
+                            max_topic_alias,
+                            max_qos,
+                            pool.clone(),
+                        )
+                    },
+                ))
             }
         }),
     )
@@ -366,12 +367,12 @@ fn handshake_service_factory2<Io, St, C>(
     max_receive: u16,
     max_topic_alias: u16,
     max_qos: Option<QoS>,
-    handshake_timeout: u16,
+    handshake_timeout: Seconds,
     pool: Rc<MqttSinkPool>,
 ) -> impl ServiceFactory<
     Config = (),
     Request = (Io, State),
-    Response = (Io, State, Rc<MqttShared>, Session<St>, u16),
+    Response = (Io, State, Rc<MqttShared>, Session<St>, Seconds),
     Error = MqttError<C::Error>,
     InitError = C::InitError,
 >
@@ -380,27 +381,30 @@ where
     C: ServiceFactory<Config = (), Request = Handshake<Io>, Response = HandshakeAck<Io, St>>,
     C::Error: fmt::Debug,
 {
-    ntex::apply(
-        Timeout::new(Duration::from_millis(handshake_timeout as u64)),
-        ntex::fn_factory(move || {
+    ntex::service::apply(
+        Timeout::new(Millis::from(handshake_timeout)),
+        ntex::service::fn_factory(move || {
             let pool = pool.clone();
             let fut = factory.new_service(());
             async move {
                 let service = fut.await?;
                 let pool = pool.clone();
                 let service = Rc::new(service.map_err(MqttError::Service));
-                Ok::<_, C::InitError>(ntex::apply_fn(service, move |(io, state), service| {
-                    handshake(
-                        io,
-                        Some(state),
-                        service.clone(),
-                        max_size,
-                        max_receive,
-                        max_topic_alias,
-                        max_qos,
-                        pool.clone(),
-                    )
-                }))
+                Ok::<_, C::InitError>(ntex::service::apply_fn(
+                    service,
+                    move |(io, state), service| {
+                        handshake(
+                            io,
+                            Some(state),
+                            service.clone(),
+                            max_size,
+                            max_receive,
+                            max_topic_alias,
+                            max_qos,
+                            pool.clone(),
+                        )
+                    },
+                ))
             }
         }),
     )
@@ -420,7 +424,7 @@ async fn handshake<Io, S, St, E>(
     mut max_topic_alias: u16,
     max_qos: Option<QoS>,
     pool: Rc<MqttSinkPool>,
-) -> Result<(Io, State, Rc<MqttShared>, Session<St>, u16), S::Error>
+) -> Result<(Io, State, Rc<MqttShared>, Session<St>, Seconds), S::Error>
 where
     Io: AsyncRead + AsyncWrite + Unpin + 'static,
     S: Service<Request = Handshake<Io>, Response = HandshakeAck<Io, St>, Error = MqttError<E>>,
@@ -497,7 +501,11 @@ where
 
                     state.set_buffer_params(ack.read_hw, ack.write_hw, ack.lw);
                     state
-                        .send(&mut ack.io, &shared.codec, mqtt::Packet::ConnectAck(ack.packet))
+                        .send(
+                            &mut ack.io,
+                            &shared.codec,
+                            mqtt::Packet::ConnectAck(Box::new(ack.packet)),
+                        )
                         .await?;
 
                     Ok((
@@ -510,7 +518,7 @@ where
                             max_receive,
                             max_topic_alias,
                         ),
-                        ack.keepalive,
+                        Seconds(ack.keepalive),
                     ))
                 }
                 None => {
@@ -521,7 +529,10 @@ where
                             .shared
                             .state
                             .write()
-                            .encode(mqtt::Packet::ConnectAck(ack.packet), &ack.shared.codec)
+                            .encode(
+                                mqtt::Packet::ConnectAck(Box::new(ack.packet)),
+                                &ack.shared.codec,
+                            )
                             .is_ok()
                     {
                         WriteTask::shutdown(
@@ -552,7 +563,7 @@ pub(crate) struct ServerSelector<St, C, T, Io, F, R> {
     max_size: u32,
     max_receive: u16,
     max_qos: Option<QoS>,
-    disconnect_timeout: u16,
+    disconnect_timeout: Seconds,
     max_topic_alias: u16,
     _t: marker::PhantomData<(St, Io, R)>,
 }
@@ -560,7 +571,7 @@ pub(crate) struct ServerSelector<St, C, T, Io, F, R> {
 impl<St, C, T, Io, F, R> ServiceFactory for ServerSelector<St, C, T, Io, F, R>
 where
     Io: AsyncRead + AsyncWrite + Unpin + 'static,
-    F: Fn(&mqtt::Connect) -> R + 'static,
+    F: Fn(&Handshake<Io>) -> R + 'static,
     R: Future<Output = Result<bool, C::Error>>,
     C: ServiceFactory<Config = (), Request = Handshake<Io>, Response = HandshakeAck<Io, St>>
         + 'static,
@@ -617,7 +628,7 @@ pub(crate) struct ServerSelectorImpl<St, C, T, Io, F, R> {
     max_size: u32,
     max_receive: u16,
     max_qos: Option<QoS>,
-    disconnect_timeout: u16,
+    disconnect_timeout: Seconds,
     max_topic_alias: u16,
     time: Timer,
     _t: marker::PhantomData<(St, Io, R)>,
@@ -626,7 +637,7 @@ pub(crate) struct ServerSelectorImpl<St, C, T, Io, F, R> {
 impl<St, C, T, Io, F, R> Service for ServerSelectorImpl<St, C, T, Io, F, R>
 where
     Io: AsyncRead + AsyncWrite + Unpin + 'static,
-    F: Fn(&mqtt::Connect) -> R + 'static,
+    F: Fn(&Handshake<Io>) -> R + 'static,
     R: Future<Output = Result<bool, C::Error>>,
     C: Service<Request = Handshake<Io>, Response = HandshakeAck<Io, St>> + 'static,
     C::Error: fmt::Debug,
@@ -668,39 +679,37 @@ where
         let mut max_topic_alias = self.max_topic_alias;
 
         Box::pin(async move {
-            let (pkt, io, state, shared, mut delay) = req;
+            let (mut hnd, state, mut delay) = req;
 
             let result = if let Some(ref mut delay) = delay {
-                let fut = (&*check)(&pkt);
+                let fut = (&*check)(&hnd);
                 match crate::utils::select(fut, delay).await {
                     Either::Left(res) => res,
                     Either::Right(_) => return Err(MqttError::HandshakeTimeout),
                 }
             } else {
-                (&*check)(&pkt).await
+                (&*check)(&hnd).await
             };
 
             if !result.map_err(MqttError::Service)? {
-                Ok(Either::Left((pkt, io, state, shared, delay)))
+                Ok(Either::Left((hnd, state, delay)))
             } else {
                 // set max outbound (encoder) packet size
-                if let Some(size) = pkt.max_packet_size {
-                    shared.codec.set_max_outbound_size(size.get());
+                if let Some(size) = hnd.packet().max_packet_size {
+                    hnd.shared.codec.set_max_outbound_size(size.get());
                 }
-                shared.cap.set(pkt.receive_max.map(|v| v.get()).unwrap_or(16) as usize);
+                hnd.shared
+                    .cap
+                    .set(hnd.packet().receive_max.map(|v| v.get()).unwrap_or(16) as usize);
 
-                let keep_alive = pkt.keep_alive;
+                let keep_alive = hnd.packet().keep_alive;
+                hnd.max_size = max_size;
+                hnd.max_receive = max_receive;
+                hnd.max_topic_alias = max_topic_alias;
 
                 // authenticate mqtt connection
                 let mut ack = if let Some(ref mut delay) = delay {
-                    let fut = connect.call(Handshake::new(
-                        pkt,
-                        io,
-                        shared,
-                        max_size,
-                        max_receive,
-                        max_topic_alias,
-                    ));
+                    let fut = connect.call(hnd);
                     match crate::utils::select(fut, delay).await {
                         Either::Left(res) => res.map_err(|e| {
                             log::trace!("Connection handshake failed: {:?}", e);
@@ -709,20 +718,10 @@ where
                         Either::Right(_) => return Err(MqttError::HandshakeTimeout),
                     }
                 } else {
-                    connect
-                        .call(Handshake::new(
-                            pkt,
-                            io,
-                            shared,
-                            max_size,
-                            max_receive,
-                            max_topic_alias,
-                        ))
-                        .await
-                        .map_err(|e| {
-                            log::trace!("Connection handshake failed: {:?}", e);
-                            MqttError::Service(e)
-                        })?
+                    connect.call(hnd).await.map_err(|e| {
+                        log::trace!("Connection handshake failed: {:?}", e);
+                        MqttError::Service(e)
+                    })?
                 };
 
                 match ack.session {
@@ -755,7 +754,7 @@ where
                             .send(
                                 &mut ack.io,
                                 &shared.codec,
-                                mqtt::Packet::ConnectAck(ack.packet),
+                                mqtt::Packet::ConnectAck(Box::new(ack.packet)),
                             )
                             .await?;
 
@@ -769,7 +768,7 @@ where
                         log::trace!("Connection handler is created, starting dispatcher");
 
                         Dispatcher::with(ack.io, shared.state.clone(), shared, handler, time)
-                            .keepalive_timeout(ack.keepalive as u16)
+                            .keepalive_timeout(Seconds(ack.keepalive))
                             .disconnect_timeout(timeout)
                             .await?;
                         Ok(Either::Right(()))
@@ -782,7 +781,10 @@ where
                                 .shared
                                 .state
                                 .write()
-                                .encode(mqtt::Packet::ConnectAck(ack.packet), &ack.shared.codec)
+                                .encode(
+                                    mqtt::Packet::ConnectAck(Box::new(ack.packet)),
+                                    &ack.shared.codec,
+                                )
                                 .is_ok()
                         {
                             WriteTask::shutdown(
