@@ -3,8 +3,8 @@ use std::{num::NonZeroU16, time::Duration};
 
 use futures::{future::ok, FutureExt, SinkExt, StreamExt};
 use ntex::codec::Framed;
-use ntex::rt::time::sleep;
 use ntex::server;
+use ntex::time::{sleep, Seconds};
 use ntex::util::{poll_fn, ByteString, Bytes};
 
 use ntex_mqtt::v3::{
@@ -18,7 +18,7 @@ async fn handshake<Io>(mut packet: Handshake<Io>) -> Result<HandshakeAck<Io, St>
     packet.packet_mut();
     packet.io();
     packet.sink();
-    Ok(packet.ack(St, false).idle_timeout(16))
+    Ok(packet.ack(St, false).idle_timeout(Seconds(16)))
 }
 
 #[ntex::test]
@@ -123,7 +123,7 @@ async fn test_ping() -> std::io::Result<()> {
     let io = srv.connect().await.unwrap();
     let mut framed = Framed::new(io, codec::Codec::default());
     framed
-        .send(codec::Packet::Connect(codec::Connect::default().client_id("user")))
+        .send(codec::Packet::Connect(codec::Connect::default().client_id("user").into()))
         .await
         .unwrap();
     framed.next().await.unwrap().unwrap();
@@ -157,10 +157,7 @@ async fn test_ack_order() -> std::io::Result<()> {
 
     let io = srv.connect().await.unwrap();
     let mut framed = Framed::new(io, codec::Codec::default());
-    framed
-        .send(codec::Packet::Connect(codec::Connect::default().client_id("user")))
-        .await
-        .unwrap();
+    framed.send(codec::Connect::default().client_id("user").into()).await.unwrap();
     let _ = framed.next().await.unwrap().unwrap();
 
     framed
@@ -249,8 +246,8 @@ async fn test_ack_order_sink() -> std::io::Result<()> {
 async fn test_disconnect() -> std::io::Result<()> {
     let srv = server::test_server(|| {
         MqttServer::new(handshake)
-            .publish(ntex::fn_factory_with_config(|session: Session<St>| {
-                ok(ntex::fn_service(move |_: Publish| {
+            .publish(ntex::service::fn_factory_with_config(|session: Session<St>| {
+                ok(ntex::service::fn_service(move |_: Publish| {
                     session.sink().force_close();
                     async {
                         sleep(Duration::from_millis(100)).await;
@@ -306,7 +303,7 @@ async fn test_handle_incoming() -> std::io::Result<()> {
 
     let io = srv.connect().await.unwrap();
     let mut framed = Framed::new(io, codec::Codec::default());
-    framed.write(codec::Packet::Connect(codec::Connect::default().client_id("user"))).unwrap();
+    framed.write(codec::Connect::default().client_id("user").into()).unwrap();
     framed
         .write(
             codec::Publish {
